#!/usr/bin/python2

import sys
import os
import subprocess
import wx
import string

layout=[]
#[ name, focus-or-hide, stack-level ]
layers={}
#[ name, geometry, force-maximize, sticky, skip-focus, max-clients ]
tiles={}
dpysize=(1920,1080)

def find_layer(name):
    for i,lay in enumerate(layout):
        if lay[0]==name:
            return i
    return -1

def find_tile(name,layer_index):
    for i,til in enumerate(layout[layer_index][1]):
        if til==name:
            return i
    return -1

def geom(strng):
    if '%' in strng:
        (rel,ab)=strng.split('%')
        if ab:
            return [float(rel),int(ab)]
        else:
            return [float(rel),0]
    else:
        return [0.0,int(strng)]

#returns layer-name
def decode_layer(name):
    if name.isdigit():
        index=int(name)
        if index<=0 or index>len(layout):
            return None
        return layout[index-1][0]
    else:
        return name

#returns (layer-name, tile-name, full-name)
def decode_tile(name):
    if '.' in name:
        (layer,tile)=name.split('.')
        lay_n=decode_layer(layer)
        if not lay_n:
            return (None,None,None)
        if tile.isdigit():
            til_i=int(tile)
            if til_i<0 or til_i>=len(layout[lay_i][1]):
                return (None,None,None)
            til_n=layout[find_layer(lay_n)][1][til_i]
            return (lay_n,til_n,'.'.join((lay_n,til_n)))
        return (layer,tile,'.'.join((layer,tile)))
    else:
        for i,l in enumerate(layout):
            for j,t in enumerate(l[1]):
                if t==tile:
                    return (l[0],t,'.'.join((l[0],t)))
    return (None,None,None)

str_stcklvl={"bottom":-1, "normal":0, "top":1}
istr_stcklvl=dict(zip(str_stcklvl.values(), str_stcklvl.keys()))
str_yesno={"yes":True, "no":False}
str_sticks={"sticky":-1, "normal":0, "slippery":1, "avoid":2}
istr_sticks=dict(zip(str_sticks.values(), str_sticks.keys()))

def savelayout(filename=None):
    if filename:
        f=open(filename,"w")
    else:
        f=sys.stdout

    for layer in layout:
        f.write('# layer {} specification\n'.format(layer[0]))
        f.write('layer-add {}\n'.format(layer[0]))
        if layers[layer[0]][1]:
            f.write('layer-set {} focus-or-hide yes\n'.format(layer[0]))
        if layers[layer[0]][2]:
            f.write('layer-set {} stack-level {}\n'.format(layer[0],istr_stcklvl[layers[layer[0]][2]]))

        f.write('# tiles of layer {}\n'.format(layer[0]))
        for tile in layer[1]:
            f.write('tile-add {0} {1}%{2:+d} {3}%{4:+d} {5}%{6:+d} {7}%{8:+d}\n'.format(tile,*[int(x) for x in tiles[tile][1]]))
            if tiles[tile][2]:
                f.write('tile-set {0} force-maximize yes\n'.format(tile))
            if tiles[tile][3]:
                f.write('tile-set {0} sticky {1}\n'.format(tile,istr_sticks[tiles[tile][3]]))
            if tiles[tile][4]:
                f.write('tile-set {0} skip-focus yes\n'.format(tile))
            if tiles[tile][5]:
                f.write('tile-set {0} max-clients {1}\n'.format(tile,tiles[tile][5]))
        f.write('\n')

    if filename:
        f.close()

def loadlayout(filename=os.path.expanduser("~/.wingedmonkeyrc")):
    global layout
    global tiles
    global layers
    layout=[]
    tiles={}
    layers={}
    if not os.path.exists(filename):
        return
    f=open(filename,"r")
    for line in f.readlines():
        if line[0].startswith('#'):
            continue
        args=line.split()
        if(len(args)==0):
            continue
        if args[0]=="layer-add":
            layout.append([args[1],[]])
            layers[args[1]]=[args[1],False,0]
        elif args[0]=="layer-remove":
            lay_n=decode_layer(args[1])
            if not lay_n:
                continue
            lay_i=find_layer(lay_n)
            if lay_i==-1:
                continue
            del layout[lay_i]
            del layers[lay_n]
        elif args[0]=="layer-set":
            lay_n=decode_layer(args[1])
            if not lay_n:
                continue
            if args[2]=="focus-or-hide":
                layers[lay_n][1]=str_yesno.get(args[3],not layers[lay_n][1])
            elif args[2]=="stack-level":
                layers[lay_n][2]=str_stcklvl.get(args[3],0)
        elif args[0]=="tile-add":
            (lay_n,til_n,key)=decode_tile(args[1])
            if not til_n:
                continue
            lay_i=find_layer(lay_n)
            if lay_i==-1:
                continue
            layout[lay_i][1].append(key)
            tiles[key]=[til_n,geom(args[2])+geom(args[3])+geom(args[4])+geom(args[5]), False, False, False, 0 ]
        elif args[0]=="tile-remove":
            (lay_n,til_n,key)=decode_tile(args[1])
            if not til_n:
                continue
            lay_i=find_layer(lay_n)
            if lay_i==-1:
                continue
            til_i=find_tile(key,lay_i)
            if til_i==-1:
                continue
            del layout[lay_i][1][til_i]
            del tiles[key]
        elif args[0]=="tile-set":
            (lay_n,til_n,key)=decode_tile(args[1])
            if not til_n:
                continue
            lay_i=find_layer(lay_n)
            if lay_i==-1:
                continue
            til_i=find_tile(key,lay_i)
            if til_i==-1:
                continue
            if args[2]=="force-maximize":
                tiles[key][2]=str_yesno.get(args[3],not tiles[key][2])
            elif args[2]=="sticky":
                tiles[key][3]=str_sticks.get(args[3],"normal")
            elif args[2]=="skip-focus":
                tiles[key][4]=str_yesno.get(args[3],not tiles[key][4])
            elif args[2]=="max-clients":
                tiles[key][5]=int(args[3])

    f.close()


class MyPopupMenu(wx.Menu):
    
    def __init__(self, parent, is_tile, index, count):
        super(MyPopupMenu, self).__init__()
        self.parent = parent
        self.is_tile = is_tile
        self.index = index
        self.count = count

        if is_tile:
            newstrng="New Tile"
        else:
            newstrng="New Layer"

        if index>=0:
            mi_rename = wx.MenuItem(self, wx.NewId(), 'Rename')
            self.AppendItem(mi_rename)
            self.Bind(wx.EVT_MENU, self.OnRename, mi_rename)

            mi_delete = wx.MenuItem(self, wx.NewId(), 'Remove')
            self.AppendItem(mi_delete)
            self.Bind(wx.EVT_MENU, self.OnDelete, mi_delete)

            self.AppendSeparator()

            mi_moveup = wx.MenuItem(self, wx.NewId(), 'Move Up')
            self.AppendItem(mi_moveup)
            if index>0:
                self.Bind(wx.EVT_MENU, self.OnMoveup, mi_moveup)
            else:
                mi_moveup.Enable(False)

            mi_movedn = wx.MenuItem(self, wx.NewId(), 'Move Down')
            self.AppendItem(mi_movedn)
            if index<count-1:
                self.Bind(wx.EVT_MENU, self.OnMovedn, mi_movedn)
            else:
                mi_movedn.Enable(False)

            self.AppendSeparator()

        mi_new = wx.MenuItem(self, wx.NewId(), newstrng)
        self.AppendItem(mi_new)
        self.Bind(wx.EVT_MENU, self.OnNew, mi_new)


    def OnRename(self, e):
        if self.is_tile:
            self.parent.tile_rename(self.index)
        else:
            self.parent.layer_rename(self.index)

    def OnDelete(self, e):
        if self.is_tile:
            self.parent.tile_delete(self.index)
        else:
            self.parent.layer_delete(self.index)

    def OnMoveup(self, e):
        if self.is_tile:
            self.parent.tile_move(self.index,True)
        else:
            self.parent.layer_move(self.index,True)

    def OnMovedn(self, e):
        if self.is_tile:
            self.parent.tile_move(self.index,False)
        else:
            self.parent.layer_move(self.index,False)

    def OnNew(self, e):
        if self.is_tile:
            self.parent.tile_create()
        else:
            self.parent.layer_create()

class wingframe(wx.Frame):
    def __init__(self, *args, **kwargs):
        super(wingframe,self).__init__(*args, **kwargs) 

        self.movedata=None
        self.resizedata=None

        self.layer_selected=-1
        self.tile_selected=-1

        self.filename=os.path.expanduser("~/.wingedmonkeyrc")
        if not os.path.exists(self.filename):
            self.filename=""

        self.InitUI()

        self.button_import.Bind(wx.EVT_BUTTON,self.import_click)
        self.button_export.Bind(wx.EVT_BUTTON,self.export_click)

        self.list_layers.Bind(wx.EVT_LISTBOX,self.listlayer_click)
        self.list_tiles.Bind(wx.EVT_LISTBOX,self.listtiles_click)
        self.layerlist_refresh()
        self.layerlist_changesel()

        self.ly_cb_foh.Bind(wx.EVT_CHECKBOX,self.layerprops_click)
        self.ly_combo.Bind(wx.EVT_COMBOBOX,self.layerprops_click)

        self.ti_cb_fm.Bind(wx.EVT_CHECKBOX,self.tileprops_click)
        self.ti_cb_sf.Bind(wx.EVT_CHECKBOX,self.tileprops_click)
        self.ti_sticky.Bind(wx.EVT_COMBOBOX,self.tileprops_click)
        self.ti_maxcl.Bind(wx.EVT_SPINCTRL,self.tileprops_click)
        for i in range(4):
            self.g_relat[i].Bind(wx.EVT_SPINCTRL,self.tileprops_click)
            self.g_abs[i].Bind(wx.EVT_SPINCTRL,self.tileprops_click)

        self.panel_right.Bind(wx.EVT_PAINT,self.OnPaint)
        self.panel_right.Bind(wx.EVT_SIZE,self.OnPaint)

        self.list_layers.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDownList)
        self.list_tiles.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDownList)
        self.panel_right.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.panel_right.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.panel_right.Bind(wx.EVT_MOTION, self.OnLeftMove)
        self.panel_right.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
        self.panel_right.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)

        self.panel_right.Bind(wx.EVT_ERASE_BACKGROUND,self.Nothing)


    def layerprops_click(self,event):
        if self.layer_selected<0:
            return
        layername=layout[self.layer_selected][0]
        layers[layername][1]=self.ly_cb_foh.GetValue()
        layers[layername][2]=str_stcklvl[self.ly_combo.GetValue()]

        self.draw(False)

    def tileprops_click(self,event):
        if self.layer_selected<0 or self.tile_selected<0:
            return
        tilename=layout[self.layer_selected][1][self.tile_selected]
        tiles[tilename][1]=[widg.GetValue() for i in range(4) for widg in (self.g_relat[i], self.g_abs[i])]
        tiles[tilename][2]=self.ti_cb_fm.GetValue()
        tiles[tilename][3]=str_sticks[self.ti_sticky.GetValue()]
        tiles[tilename][4]=self.ti_cb_sf.GetValue()
        tiles[tilename][5]=self.ti_maxcl.GetValue()

        self.draw(False)


    #change selection of layer, which must refresh the tile list and the properties
    def layerlist_changesel(self):
        if self.layer_selected<0:
            self.ly_cb_foh.Enable(False)
            self.ly_combo.Enable(False)
            self.tile_selected=-1
        else:
            layername=layout[self.layer_selected][0]
            self.ly_cb_foh.Enable()
            self.ly_combo.Enable()
            self.ly_cb_foh.SetValue(layers[layername][1])
            self.ly_combo.SetValue(istr_stcklvl[layers[layername][2]])
            self.tile_selected=0
        
        self.tilelist_refresh()
        self.tilelist_changesel()

    def tilelist_changesel(self):
        if self.layer_selected<0 or self.tile_selected<0:
            for i in range(4):
                self.g_relat[i].Enable(False)
                self.g_abs[i].Enable(False)
            self.ti_cb_fm.Enable(False)
            self.ti_sticky.Enable(False)
            self.ti_cb_sf.Enable(False)
            self.ti_maxcl.Enable(False)
        else:
            tilename=layout[self.layer_selected][1][self.tile_selected]
            for i in range(4):
                self.g_relat[i].Enable()
                self.g_abs[i].Enable()
            self.ti_cb_fm.Enable()
            self.ti_sticky.Enable()
            self.ti_cb_sf.Enable()
            self.ti_maxcl.Enable()
            for i in range(4):
                self.g_relat[i].SetValue(tiles[tilename][1][2*i])
                self.g_abs[i].SetValue(tiles[tilename][1][2*i+1])
            self.ti_cb_fm.SetValue(tiles[tilename][2])
            self.ti_sticky.SetValue(istr_sticks[tiles[tilename][3]])
            self.ti_cb_sf.SetValue(tiles[tilename][4])
            self.ti_maxcl.SetValue(tiles[tilename][5])

        self.draw(False)

    #refresh the list of layers
    def layerlist_refresh(self):
        self.list_layers.Set([x[0] for x in layout])
        newcount=self.list_layers.GetCount()
        #fix if not in range
        if self.layer_selected>=newcount:
            self.layer_selected=newcount-1
            self.layerlist_changesel() #change selection
        self.list_layers.SetSelection(self.layer_selected)


    def tilelist_refresh(self):
        if self.layer_selected<0:
            self.list_tiles.Set([])
        else:
            self.list_tiles.Set([tiles[x][0] for x in layout[self.layer_selected][1]])
        newcount=self.list_tiles.GetCount()
        #fix if not in range
        if self.tile_selected>=newcount:
            self.tile_selected=newcount-1
            self.tilelist_changesel()
        self.list_tiles.SetSelection(self.tile_selected)

    def listlayer_click(self,event):
        sel=self.list_layers.GetSelection()
        if sel==wx.NOT_FOUND:
            return
        self.layer_selected=sel
        self.layerlist_changesel()

    def listtiles_click(self,event):
        selt=self.list_tiles.GetSelection()
        if selt==wx.NOT_FOUND:
            return
        self.tile_selected=selt
        self.tilelist_changesel()        

    def import_click(self,evt):
        dialg=wx.FileDialog(self,"Load wingedmonkey layout file",os.path.dirname(self.filename),os.path.basename(self.filename),style=wx.FD_OPEN)
        if dialg.ShowModal()!=wx.ID_OK:
            return
        newfile=dialg.GetPath()
        if os.path.exists(newfile):
            loadlayout(newfile)
            self.filename=newfile
        self.layer_selected=0
        self.tile_selected=0
        self.layerlist_refresh()
        self.layerlist_changesel()

    def export_click(self,evt):
        dialg=wx.FileDialog(self,"Save wingedmonkey layout file",os.path.dirname(self.filename),os.path.basename(self.filename),style=wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)
        if dialg.ShowModal()!=wx.ID_OK:
            return
        newfile=dialg.GetPath()
        savelayout(newfile)
        self.filename=newfile

    def InitUI(self):
        self.panel_main=wx.Panel(self,size=(-1,-1))
        #set up sizers
        self.sizer_horz=wx.BoxSizer(wx.HORIZONTAL)
        self.sizer_left=wx.GridBagSizer(3,3)

        #panels
        self.panel_left=wx.Panel(self.panel_main,-1,style=wx.SIMPLE_BORDER,size=(-1,-1))
        self.panel_right=wx.Panel(self.panel_main,-1,style=wx.SIMPLE_BORDER,size=(500,500*dpysize[1]/dpysize[0]))
        self.panel_right.SetMinSize((0,0))
        self.panel_right.SetToolTipString('screen size: {}x{}'.format(*dpysize))
        self.sizer_horz.Add(self.panel_left,0,flag=wx.EXPAND)
        self.sizer_horz.Add(self.panel_right,1,flag=wx.SHAPED)

        #buttons
        self.button_import=wx.Button(self.panel_left,-1,"Import",size=(128,32))
        self.button_export=wx.Button(self.panel_left,-1,"Export",size=(128,32))
        self.sizer_left.Add(self.button_import,pos=(0,0),flag=wx.EXPAND)
        self.sizer_left.Add(self.button_export,pos=(0,1),flag=wx.EXPAND)

        #decor
        self.sizer_left.Add(wx.StaticLine(self.panel_left,style=wx.LI_HORIZONTAL),pos=(1,0),flag=wx.EXPAND)
        self.sizer_left.Add(wx.StaticLine(self.panel_left,style=wx.LI_HORIZONTAL),pos=(1,1),flag=wx.EXPAND)


        self.dec_layers=wx.StaticText(self.panel_left,-1,"Layers:",style=wx.ALIGN_LEFT)
        self.dec_tiles=wx.StaticText(self.panel_left,-1,"Tiles:",style=wx.ALIGN_LEFT)
        self.sizer_left.Add(self.dec_layers,pos=(2,0),flag=wx.EXPAND)
        self.sizer_left.Add(self.dec_tiles,pos=(2,1),flag=wx.EXPAND)



	#listbox
        self.list_layers=wx.ListBox(self.panel_left,-1,style=wx.LB_SINGLE,size=(-1,-1))
        self.list_layers.SetMinSize((-1,0))
        self.list_tiles=wx.ListBox(self.panel_left,-1,style=wx.LB_SINGLE,size=(-1,-1))
        self.list_tiles.SetMinSize((-1,0))

        self.sizer_left.Add(self.list_layers,pos=(3,0),flag=wx.EXPAND)
        self.sizer_left.Add(self.list_tiles,pos=(3,1),flag=wx.EXPAND)

        self.sizer_left.AddGrowableCol(0,0)
        self.sizer_left.AddGrowableCol(1,0)
        self.sizer_left.AddGrowableRow(3,0)

        #properties layers
        self.ly_props=wx.Panel(self.panel_left,-1,(-1,-1))

        self.ly_labl_cb_foh=wx.StaticText(self.ly_props,-1,"focus-or-hide:",style=wx.ALIGN_LEFT)
        self.ly_labl_combo=wx.StaticText(self.ly_props,-1,"stack-level:",style=wx.ALIGN_LEFT)

        self.ly_cb_foh=wx.CheckBox(self.ly_props,-1,"",style=wx.ALIGN_RIGHT)
        self.ly_combo=wx.ComboBox(self.ly_props,-1,"normal",style=wx.CB_DROPDOWN|wx.CB_READONLY,choices=str_stcklvl.keys())
        self.ly_combo.SetMinSize((100,-1))

        self.sizer_lyprops=wx.FlexGridSizer(2,2,2,20)
        self.sizer_lyprops.Add(self.ly_labl_cb_foh,1,flag=wx.ALIGN_LEFT)
        self.sizer_lyprops.Add(self.ly_cb_foh,0,flag=wx.ALIGN_RIGHT)
        self.sizer_lyprops.Add(self.ly_labl_combo,1,flag=wx.ALIGN_LEFT)
        self.sizer_lyprops.Add(self.ly_combo,0,flag=wx.ALIGN_RIGHT)

        self.sizer_lyprops.AddGrowableCol(1)
        self.ly_props.SetSizer(self.sizer_lyprops)

        self.sizer_left.Add(self.ly_props,pos=(4,0),span=(2,1),flag=wx.EXPAND)

        #tile geometry
        self.ti_geom=wx.Panel(self.panel_left,-1,(-1,-1))

        self.g_labels=[wx.StaticText(self.ti_geom,-1,x) for x in ("x=","y=","w=","h=")]
        self.g_relat=[wx.SpinCtrl(self.ti_geom,-1,min=-200,max=200) for x in range(4)]
        self.g_abs=[wx.SpinCtrl(self.ti_geom,-1,min=-4096,max=4096) for x in range(4)]

        widgs=[lst[x] for x in range(4) for lst in (self.g_labels,self.g_relat,self.g_abs)]

        self.g_relat[0].SetToolTipString("x offset in screen width percentage")
        self.g_relat[1].SetToolTipString("y offset in screen height percentage")
        self.g_relat[2].SetToolTipString("tile width in screen width percentage")
        self.g_relat[3].SetToolTipString("tile height in screen height percentage")

        self.g_abs[0].SetToolTipString("additional x offset in pixels")
        self.g_abs[1].SetToolTipString("additional y offset in pixels")
        self.g_abs[2].SetToolTipString("additional tile width in pixels")
        self.g_abs[3].SetToolTipString("additional tile height in pixels")


        self.sizer_geom=wx.FlexGridSizer(4,3,2,4)
        self.sizer_geom.AddGrowableCol(1)
        self.sizer_geom.AddGrowableCol(2)

        for widgt in widgs:
            self.sizer_geom.Add(widgt,flag=wx.EXPAND)

        self.ti_geom.SetSizer(self.sizer_geom)

        self.sizer_left.Add(self.ti_geom,pos=(4,1),span=(4,1),flag=wx.EXPAND)

        #properties tiles
        self.ti_props=wx.Panel(self.panel_left,-1,(-1,-1))

        self.ti_labl_cb_fm=wx.StaticText(self.ti_props,-1,"force-maximize:",style=wx.ALIGN_LEFT)
        self.ti_labl_cb_sf=wx.StaticText(self.ti_props,-1,"skip-focus:",style=wx.ALIGN_LEFT)
        self.ti_labl_sticky=wx.StaticText(self.ti_props,-1,"sticky:",style=wx.ALIGN_LEFT)
        self.ti_labl_maxcl=wx.StaticText(self.ti_props,-1,"max-clients:",style=wx.ALIGN_LEFT)

        
        self.ti_cb_fm=wx.CheckBox(self.ti_props,-1,"",style=wx.ALIGN_RIGHT)
        self.ti_cb_sf=wx.CheckBox(self.ti_props,-1,"",style=wx.ALIGN_RIGHT)
        self.ti_sticky=wx.ComboBox(self.ti_props,-1,"normal",style=wx.CB_DROPDOWN|wx.CB_READONLY,choices=str_sticks.keys())
        self.ti_maxcl=wx.SpinCtrl(self.ti_props,-1,style=wx.SP_ARROW_KEYS,min=0,name="max-clients")

        self.ti_sticky.SetMinSize((100,-1))
        self.ti_maxcl.SetMinSize((100,-1))


        self.sizer_tiprops=wx.FlexGridSizer(2,2,2,20)
        self.sizer_tiprops.AddGrowableCol(1)

        self.sizer_tiprops.Add(self.ti_labl_cb_fm,flag=wx.ALIGN_LEFT)
        self.sizer_tiprops.Add(self.ti_cb_fm,flag=wx.ALIGN_RIGHT)
        self.sizer_tiprops.Add(self.ti_labl_cb_sf,flag=wx.ALIGN_LEFT)
        self.sizer_tiprops.Add(self.ti_cb_sf,flag=wx.ALIGN_RIGHT)
        self.sizer_tiprops.Add(self.ti_labl_sticky,flag=wx.ALIGN_LEFT)
        self.sizer_tiprops.Add(self.ti_sticky,flag=wx.ALIGN_RIGHT)
        self.sizer_tiprops.Add(self.ti_labl_maxcl,flag=wx.ALIGN_LEFT)
        self.sizer_tiprops.Add(self.ti_maxcl,flag=wx.ALIGN_RIGHT)

        self.ti_props.SetSizer(self.sizer_tiprops)

        self.sizer_left.Add(self.ti_props,pos=(8,1),span=(4,1),flag=wx.EXPAND)

        #connect stuff
        self.panel_main.SetSizer(self.sizer_horz)
        self.panel_left.SetSizer(self.sizer_left)


        self.list_layers.Set([x[0] for x in layout])


        self.Refresh()

    def OnPaint(self, e):
        self.draw(True)

    def Nothing(self,e):
        return

    def draw(self,paintevt):
        if paintevt:
            pdc=wx.PaintDC(self.panel_right)
        else:
            pdc=wx.ClientDC(self.panel_right)
        dc = wx.BufferedDC(pdc)

        dc.SetPen(wx.Pen('WHITE'))
        dc.SetBrush(wx.Brush('WHITE'))
        dc.DrawRectangle(0,0,*self.panel_right.GetSize())

        sel=self.layer_selected
        if sel<0:
            return
        tsel=self.tile_selected
        for indx,tile in enumerate(layout[sel][1]):
            if indx==tsel:
                continue
            self.draw_tile(tiles[tile],dc,False)
        if tsel<0:
            return
        self.draw_tile(tiles[layout[sel][1][tsel]],dc,True)

    def draw_tile(self,tilespec,dc,selected):
        rescale=float(self.panel_right.GetSize()[0])/dpysize[0]
        if tilespec[2]:
            clr=(50,25,100)
            bclr=(100,50,200)
        else:
            clr=(100,50,25)
            bclr=(200,100,50)
        if not selected:
            bclr=tuple([255-(255-x)/2 for x in bclr])
        mypen=wx.Pen(clr)
        mybrsh=wx.Brush(bclr)
        if tilespec[4]:
            mypen.SetStyle(wx.SHORT_DASH)

        mypen.SetJoin(wx.JOIN_ROUND)
        mypen.SetCap(wx.CAP_ROUND)
        geomrect=[dpysize[0]*tilespec[1][0]*0.01+tilespec[1][1], \
                  dpysize[1]*tilespec[1][2]*0.01+tilespec[1][3], \
                  dpysize[0]*tilespec[1][4]*0.01+tilespec[1][5], \
                  dpysize[1]*tilespec[1][6]*0.01+tilespec[1][7]]
        geomrect=[x*rescale for x in geomrect]
        dc.SetPen(mypen)
        dc.SetBrush(mybrsh)
        dc.DrawRoundedRectangle(geomrect[0],geomrect[1],geomrect[2],geomrect[3],5)
        dc.SetPen(wx.Pen('BLACK'))
        if tilespec[5]:
            thetext='{} [{}]'.format(tilespec[0],tilespec[5])
        else:
            thetext=tilespec[0]
        tw,th=dc.GetTextExtent(thetext)
        dc.DrawText(thetext,geomrect[0]+0.5*geomrect[2]-tw/2,geomrect[1]+0.5*geomrect[3]-th/2)
        if self.movedata and selected:
            dc.SetPen(wx.Pen('BLACK'))
            xr=dpysize[0]*tilespec[1][0]*0.01*rescale
            yr=dpysize[1]*tilespec[1][2]*0.01*rescale
            dc.DrawLine(xr,0,xr,dc.GetSize()[1])
            dc.DrawLine(0,yr,dc.GetSize()[0],yr)

            if self.movedata[3]:
                dc.SetBrush(wx.Brush('RED'))
                dc.DrawCircle(geomrect[0],geomrect[1],3)
                dc.SetPen(wx.Pen('RED'))
                dc.DrawLine(xr,yr,geomrect[0],geomrect[1])
            else:
                dc.SetBrush(wx.Brush('RED'))
                dc.DrawCircle(xr,yr,3)
                dc.SetPen(wx.Pen('BLACK'))
                dc.DrawLine(xr,yr,geomrect[0],geomrect[1])

        elif self.resizedata and selected:
            dc.SetPen(wx.Pen('BLACK'))
            wr=dpysize[0]*tilespec[1][4]*0.01*rescale
            hr=dpysize[1]*tilespec[1][6]*0.01*rescale
            dc.SetBrush(wx.Brush('BLACK',wx.TRANSPARENT))
            dc.DrawRectangle(geomrect[0],geomrect[1],wr,hr)
            dc.DrawLine(geomrect[0],geomrect[1],geomrect[0]+wr,geomrect[1]+hr)
            dc.DrawLine(geomrect[0]+wr,geomrect[1],geomrect[0],geomrect[1]+hr)

            if self.resizedata[3]:
                dc.SetBrush(wx.Brush('RED'))
                dc.DrawCircle(geomrect[0]+geomrect[2],geomrect[1]+geomrect[3],3)
                dc.SetPen(wx.Pen('RED'))
                dc.DrawLine(geomrect[0]+geomrect[2],geomrect[1]+geomrect[3],
                            geomrect[0]+wr,geomrect[1]+hr)
            else:
                dc.SetBrush(wx.Brush('RED'))
                dc.DrawCircle(geomrect[0]+wr,geomrect[1]+hr,3)
                dc.SetPen(wx.Pen('BLACK'))
                dc.DrawLine(geomrect[0]+geomrect[2],geomrect[1]+geomrect[3],
                            geomrect[0]+wr,geomrect[1]+hr)

            
    def find_tile_clicked(self,point):
        if self.layer_selected<0:
            return
        tilecnt=len(layout[self.layer_selected][1])
        rescale=float(self.panel_right.GetSize()[0])/dpysize[0]
        if self.tile_selected>=0:
            tile=layout[self.layer_selected][1][self.tile_selected]
            tilespec=tiles[tile]
            geomrect=[dpysize[0]*tilespec[1][0]*0.01+tilespec[1][1], \
                          dpysize[1]*tilespec[1][2]*0.01+tilespec[1][3], \
                          dpysize[0]*tilespec[1][4]*0.01+tilespec[1][5], \
                          dpysize[1]*tilespec[1][6]*0.01+tilespec[1][7]]
            geomrect=[x*rescale for x in geomrect]
            if wx.Rect(*geomrect).Contains(point):
                return (self.tile_selected,tile)

        for index,tile in enumerate(reversed(layout[self.layer_selected][1])):
            index=tilecnt-index-1
            tilespec=tiles[tile]
            geomrect=[dpysize[0]*tilespec[1][0]*0.01+tilespec[1][1], \
                          dpysize[1]*tilespec[1][2]*0.01+tilespec[1][3], \
                          dpysize[0]*tilespec[1][4]*0.01+tilespec[1][5], \
                          dpysize[1]*tilespec[1][6]*0.01+tilespec[1][7]]
            geomrect=[x*rescale for x in geomrect]
            if wx.Rect(*geomrect).Contains(point):
                return (index,tile)
        return (-1,None)

    def OnLeftMove(self,e):
        global tiles
        if self.movedata:
            rescale=float(self.panel_right.GetSize()[0])/dpysize[0]
            pos=e.GetPosition()
            dp=pos-self.movedata[2]
            if self.movedata[3]: #shift was down
                #move offset
                tiles[self.movedata[0]][1][1]=self.movedata[1][1]+int(dp[0]/rescale)
                tiles[self.movedata[0]][1][3]=self.movedata[1][3]+int(dp[1]/rescale)
            else:
                #move relative origin
                tiles[self.movedata[0]][1][0]=self.movedata[1][0]+int(100*dp[0]/rescale/dpysize[0])
                tiles[self.movedata[0]][1][2]=self.movedata[1][2]+int(100*dp[1]/rescale/dpysize[1])
            self.draw(False)
        elif self.resizedata:
            rescale=float(self.panel_right.GetSize()[0])/dpysize[0]
            pos=e.GetPosition()
            dp=pos-self.resizedata[2]
            if self.resizedata[3]: #shift was down
                #change absolute size
                tiles[self.resizedata[0]][1][5]=self.resizedata[1][5]+int(dp[0]/rescale)
                tiles[self.resizedata[0]][1][7]=self.resizedata[1][7]+int(dp[1]/rescale)
            else:
                #change relative size
                tiles[self.resizedata[0]][1][4]=self.resizedata[1][4]+int(100*dp[0]/rescale/dpysize[0])
                tiles[self.resizedata[0]][1][6]=self.resizedata[1][6]+int(100*dp[1]/rescale/dpysize[1])
            self.draw(False)
            

    def OnLeftUp(self,e):
        if self.movedata:
            self.movedata=None
            self.tilelist_changesel()

    def OnRightUp(self,e):
        if self.resizedata:
            self.resizedata=None
            self.tilelist_changesel()


    def OnLeftDown(self,e):
        e.Skip()
        if not e.AltDown() and not e.ControlDown():
            i,t=self.find_tile_clicked(e.GetPosition())
            if i<0:
                return
            self.tile_selected=i
            self.tilelist_changesel()
            self.list_tiles.SetSelection(self.tile_selected)
            self.movedata=(t,list(tiles[t][1]),e.GetPosition(),e.ShiftDown())

    def OnRightDown(self,e):
        e.Skip()
        if not e.AltDown() and not e.ControlDown():
            i,t=self.find_tile_clicked(e.GetPosition())
            if i<0 or i != self.tile_selected:
                return
            self.resizedata=(t,list(tiles[t][1]),e.GetPosition(),e.ShiftDown())

    def OnRightDownList(self,e):
        if e.GetEventObject()==self.list_layers:
            indx=self.list_layers.HitTest(e.GetPosition())
            if indx==wx.NOT_FOUND:
                indx=-1
            else:
                self.layer_selected=indx
                self.layerlist_changesel()
            self.PopupMenu(MyPopupMenu(self,False,indx,len(layout)), e.GetPosition()+self.list_layers.GetPosition())
        elif e.GetEventObject()==self.list_tiles:
            if self.layer_selected<0:
                return
            indx=self.list_tiles.HitTest(e.GetPosition())
            if indx==wx.NOT_FOUND:
                indx=-1
            else: #select if we hit anything
                self.tile_selected=indx
                self.tilelist_changesel()
            self.PopupMenu(MyPopupMenu(self,True,indx,len(layout[self.layer_selected][1])), e.GetPosition()+self.list_tiles.GetPosition())

    def tile_rename(self,index):
        if self.layer_selected<0:
            return
        global layout,layers,tiles
        oldkey=layout[self.layer_selected][1][index]
        oldname=tiles[oldkey][0]
        dial=wx.TextEntryDialog(self,"Enter new tile name:","Tile Rename",oldname)
        dial.ShowModal()
        newname=dial.GetValue()
        if newname==oldname:
            return
        newname=''.join([ c for c in newname if c in string.ascii_letters or c in string.digits or c in "-_"])
        newkey='.'.join((layout[self.layer_selected][0],newname))
        layout[self.layer_selected][1][index]=newkey
        tiles[newkey]=tiles[oldkey]
        tiles[newkey][0]=newname
        del tiles[oldkey]
        self.tilelist_refresh()
        self.draw(False)

    def layer_rename(self,index):
        global layout,layers,tiles
        oldname=layout[index][0]
        dial=wx.TextEntryDialog(self,"Enter new layer name:","Layer Rename",oldname)
        dial.ShowModal()
        newname=dial.GetValue()
        if newname==oldname:
            return
        newname=''.join([ c for c in newname if c in string.ascii_letters or c in string.digits or c in "-_"])
        layout[index][0]=newname
        layers[newname]=layers[oldname]
        layers[newname][0]=newname
        del layers[oldname]
        for i,tile in enumerate(layout[index][1]):
            newkey='.'.join((newname,tiles[tile][0]))
            tiles[newkey]=tiles[tile]
            del tiles[tile]
            layout[index][1][i]=newkey
        self.layerlist_refresh()
        #print layout

    def tile_delete(self,index):
        if self.layer_selected<0:
            return
        global layout,layers,tiles
        oldkey=layout[self.layer_selected][1][index]
        oldname=tiles[oldkey][0]
        del layout[self.layer_selected][1][index]
        del tiles[oldkey]
        self.tilelist_refresh()
        self.tilelist_changesel()

    def layer_delete(self,index):
        global layout,layers,tiles
        oldname=layout[index][0]
        del layers[oldname]
        for tile in layout[index][1]:
            del tiles[tile]
        del layout[index]
        self.layerlist_refresh()
        self.layerlist_changesel()
        #print layout

    def tile_create(self):
        if self.layer_selected<0:
            return
        global layout,layers,tiles
        dial=wx.TextEntryDialog(self,"Enter tile name:","Tile Create","")
        dial.ShowModal()
        newname=dial.GetValue()
        newname=''.join([ c for c in newname if c in string.ascii_letters or c in string.digits or c in "-_"])
        newkey='.'.join((layout[self.layer_selected][0],newname))
        if newkey in layout[self.layer_selected][1]:
            wx.MessageBox("Tile with that name already exists!",'Warning',wx.OK | wx.ICON_EXCLAMATION)
            return

        layout[self.layer_selected][1].append(newkey)
        tiles[newkey]=[newname,[0,0,0,0,100,0,100,0],False,False,False,0]
        self.tilelist_refresh()
        self.tilelist_changesel()

    def layer_create(self):
        global layout,layers,tiles
        dial=wx.TextEntryDialog(self,"Enter layer name:","Layer Create","")
        dial.ShowModal()
        newname=dial.GetValue()
        if newname in layers.keys():
            wx.MessageBox("Layer with that name already exists!",'Warning',wx.OK | wx.ICON_EXCLAMATION)
            return
        newname=''.join([ c for c in newname if c in string.ascii_letters or c in string.digits or c in "-_"])
        layout.append([newname,[]])
        layers[newname]=[newname,False,0]
        self.layerlist_refresh()
        self.layerlist_changesel()
        #print layout

    def tile_move(self,index,up):
        if self.layer_selected<0:
            return
        global layout,layers,tiles
        tl=layout[self.layer_selected][1]
        if up:
            tl[index],tl[index-1]=tl[index-1],tl[index]
        else:
            tl[index],tl[index+1]=tl[index+1],tl[index]

        self.tilelist_refresh()
        self.tilelist_changesel()
        #print layout

    def layer_move(self,index,up):
        if self.layer_selected<0:
            return
        global layout,layers,tiles
        if up:
            layout[index],layout[index-1]=layout[index-1],layout[index]
        else:
            layout[index],layout[index+1]=layout[index+1],layout[index]

        self.layerlist_refresh()
        self.layerlist_changesel()
        #print layout

loadlayout()

app = wx.App()
dpysize = wx.GetDisplaySize()
frame = wingframe(None,-1,'winged python',size=(900,400))
frame.Show()

app.MainLoop()
